#include "TS24501Msgs.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxCharStr.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxDiagBitTrace.h"
#include "rtxsrc/rtxError.h"
#include "rtxsrc/rtxFile.h"
#include "rtxsrc/rtxPrint.h"
#include "rt3gppsrc/rt3gppNasSec.h"

#define MAXMSGLEN 300

int main (int argc, char** argv)
{
   TS24501Msg_PDU pdu;
   TS24501Msg_ModifyRequest data;
   TS24501Msg_ModifyRequest* pvalue = &data;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN], *msgptr;
   const char*  filename = "message.dat";
   OSSIZE       len;
   int		i, ret;
   OSBOOL verbose = FALSE;
   OSCrypt128Key integKey = {
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba
   };
   OSCrypt128Key encryptKey = {
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba,
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef
   };

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else {
            printf ("usage: writer [-v]\n");
            printf ("   -v  verbose mode: print trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */
   ret = rtInitContext (&ctxt);
   if (0 != ret) {
      printf ("rtInitContext failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   ret = rtx3gppSecInitialize(&ctxt);
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ret = rtx3gppAssignAlgorithmKeys (&ctxt, &integKey, &encryptKey,
      OS3GPPSecAlgorithm_AES, OS3GPPSecAlgorithm_AES);
   if ( ret == RTERR_NOTSUPP ) {
      printf ("Warning: unable to set AES algorithm; falling back to "
              "NULL algorithm.\n");
      rtxErrReset (&ctxt);
      ret = rtx3gppAssignAlgorithmKeys (&ctxt, NULL, NULL, 
         OS3GPPSecAlgorithm_NULL_ALG, OS3GPPSecAlgorithm_NULL_ALG);
   }
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ctxt.p3gppSec->count = 15;
   ctxt.p3gppSec->bearerId = 0;
   ctxt.p3gppSec->direction = 0; /*uplink*/

   /* Populate data structure */
   asn1Init_TS24501Msg_PDU (&pdu);
   pdu.m.secHdrPresent = 1;
   pdu.secHdr.protoDiscr = TS24501Msg_NAS5GProtoDiscr_mobMgmt5G;
   pdu.secHdr.secHdrType = TS24501Msg_NAS5GSecHdrType_integProt;
   pdu.secHdr.msgAuthCode = 8000;
   pdu.secHdr.seqNumber  = 160;
   pdu.protoDiscr = TS24501Msg_NAS5GProtoDiscr_sessMgmt5G;
   pdu.hdrData.u.sm.pduSessIdent = TS24501Msg_PDUSessionIdentity_pduId6;
   pdu.hdrData.u.sm.procTransIdent = 125;

   asn1SetTC_TS24501Msg_PDU_obj_ModifyRequest5G (&ctxt, &pdu, pvalue);

   asn1Init_TS24501Msg_ModifyRequest (pvalue);

   pvalue->m.capabilityPresent = 1;
   pvalue->capability.tpmic = 0;
   pvalue->capability.atsll = 3;
   pvalue->capability.epts1 = 1;
   pvalue->capability.mh6Pdu = 0;
   pvalue->capability.rqos = 1;

   pvalue->m.maxPktFiltersPresent = 1;
   pvalue->maxPktFilters.maxFilters = 1244;

   pvalue->m.extProtoCfgOptsPresent = 1;
   pvalue->extProtoCfgOpts.octet4.configProtocol = 0;

   pvalue->m._v16ExtPresent = TRUE;
   pvalue->m.ethernetHdrCompressionCfgPresent = 1;
   pvalue->ethernetHdrCompressionCfg.cidLength = 2;

   pvalue->m._v17ExtPresent = 1;
   pvalue->m.reqMBSContainerPresent = 1;
   pvalue->reqMBSContainer.length = 0; /* length will be calculated */
   pvalue->reqMBSContainer.sessionInfo.n = 1;
   pvalue->reqMBSContainer.sessionInfo.elem[0].spare = 0;
   pvalue->reqMBSContainer.sessionInfo.elem[0].operation = TS24501IE_MBSOperations_join;
   pvalue->reqMBSContainer.sessionInfo.elem[0].type = TS24501IE_MBSSessionIdTypes_ipv4;
   {
      static const OSOCTET testdata[] = { 0xa1, 0xb2, 0xc3, 0xd4 };
      OSCRTLSAFEMEMCPY((void *)pvalue->reqMBSContainer.sessionInfo.elem[0].id.u.ipv4.srcAddrInfo,
                       4, testdata, 4);
   }
   {
      static const OSOCTET testdata[] = { 0xe5, 0xf6, 0x78, 0x90 };
      OSCRTLSAFEMEMCPY((void *)pvalue->reqMBSContainer.sessionInfo.elem[0].id.u.ipv4.destAddrInfo,
                       4, testdata, 4);
   }

   pvalue->m._v18ExtPresent = 1;
   pvalue->m.delayBudgetPresent = 1;
   pvalue->delayBudget.length = 0; /* length will be calculated */
   pvalue->delayBudget.delayBudgetList.n = 1;
   pvalue->delayBudget.delayBudgetList.elem = rtxMemAllocArrayZ(&ctxt, 1,
                                                    TS24501IE_DelayBudget);
   pvalue->delayBudget.delayBudgetList.elem[0].value = 2023;
   pvalue->delayBudget.delayBudgetList.elem[0].qfipi = 1;
   pvalue->delayBudget.delayBudgetList.elem[0].qfiInfo.u.present.numQFIs = 1;
   pvalue->delayBudget.delayBudgetList.elem[0].qfiInfo.u.present.qfiList.n = 1;
   pvalue->delayBudget.delayBudgetList.elem[0].qfiInfo.u.present.qfiList.elem = rtxMemAllocType(&ctxt, OSDynOctStr);
   pvalue->delayBudget.delayBudgetList.elem[0].qfiInfo.u.present.qfiList.elem[0].numocts = 1;
   pvalue->delayBudget.delayBudgetList.elem[0].qfiInfo.u.present.qfiList.elem[0].data = (OSOCTET *)rtxMemAlloc(&ctxt, 1);
   {
      static const OSOCTET testdata[] = { 0x3c };
      OSCRTLSAFEMEMCPY((void *)pvalue->delayBudget.delayBudgetList.elem[0].qfiInfo.u.present.qfiList.elem[0].data, 1,
                       testdata, 1);
   }

   /* Enable bit tracing */
   if (verbose) {
      rtxDiagCtxtBitFieldListInit (&ctxt);
   }

   /* Encode data */
   rtxCtxtSetBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   ret = NASEnc_TS24501Msg_PDU (&ctxt, &pdu);
   if (0 != ret) {
      printf ("encode PDU failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
      rtxFreeContext (&ctxt);
      return ret;
   }
   msgptr = rtxCtxtGetMsgPtr (&ctxt);
   len = rtxCtxtGetMsgLen (&ctxt);

   if (verbose) {
      printf ("Encoding was successful\n");
      printf ("Hex dump of encoded record:\n");
      rtxHexDump (msgptr, len);
      printf ("\n");
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
   }

   /* Verify buffer is byte-aligned */
   OSRTASSERT (ctxt.buffer.bitOffset == 8 || ctxt.buffer.bitOffset == 0);

   /* Write encoded message to output file */
   ret = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != ret) {
      printf ("rtxFileWriteBinary failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
